@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import binascii
+import contextlib
 import hashlib
 import ipaddress
 import os.path
@@ -9,15 +10,18 @@ import shutil
 import socket
 import ssl
 import tempfile
-from test import LONG_TIMEOUT, SHORT_TIMEOUT, withPyOpenSSL
+from test import LONG_TIMEOUT, SHORT_TIMEOUT, resolvesLocalhostFQDN, withPyOpenSSL
 from test.conftest import ServerConfig
 
 import pytest
 import trustme
 
 import urllib3.exceptions
-from dummyserver.server import DEFAULT_CA, HAS_IPV6, get_unreachable_address
-from dummyserver.testcase import HTTPDummyProxyTestCase, IPv6HTTPDummyProxyTestCase
+from dummyserver.socketserver import DEFAULT_CA, HAS_IPV6, get_unreachable_address
+from dummyserver.testcase import (
+    HypercornDummyProxyTestCase,
+    IPv6HypercornDummyProxyTestCase,
+)
 from urllib3 import HTTPResponse
 from urllib3._collections import HTTPHeaderDict
 from urllib3.connection import VerifiedHTTPSConnection
@@ -39,7 +43,17 @@ from urllib3.util.timeout import Timeout
 from .. import TARPIT_HOST, requires_network
 
 
-class TestHTTPProxyManager(HTTPDummyProxyTestCase):
+def assert_is_verified(pm: ProxyManager, *, proxy: bool, target: bool) -> None:
+    pool = list(pm.pools._container.values())[-1]  # retrieve last pool entry
+    connection = (
+        pool.pool.queue[-1] if pool.pool is not None else None
+    )  # retrieve last connection entry
+
+    assert connection.proxy_is_verified is proxy
+    assert connection.is_verified is target
+
+
+class TestHTTPProxyManager(HypercornDummyProxyTestCase):
     @classmethod
     def setup_class(cls) -> None:
         super().setup_class()
@@ -47,6 +61,7 @@ class TestHTTPProxyManager(HTTPDummyProxyTestCase):
         cls.http_url_alt = f"http://{cls.http_host_alt}:{int(cls.http_port)}"
         cls.https_url = f"https://{cls.https_host}:{int(cls.https_port)}"
         cls.https_url_alt = f"https://{cls.https_host_alt}:{int(cls.https_port)}"
+        cls.https_url_fqdn = f"https://{cls.https_host}.:{int(cls.https_port)}"
         cls.proxy_url = f"http://{cls.proxy_host}:{int(cls.proxy_port)}"
         cls.https_proxy_url = f"https://{cls.proxy_host}:{int(cls.https_proxy_port)}"
 
@@ -78,6 +93,30 @@ class TestHTTPProxyManager(HTTPDummyProxyTestCase):
             r = https.request("GET", f"{self.http_url}/")
             assert r.status == 200
 
+    def test_is_verified_http_proxy_to_http_target(self) -> None:
+        with proxy_from_url(self.proxy_url, ca_certs=DEFAULT_CA) as http:
+            r = http.request("GET", f"{self.http_url}/")
+            assert r.status == 200
+            assert_is_verified(http, proxy=False, target=False)
+
+    def test_is_verified_http_proxy_to_https_target(self) -> None:
+        with proxy_from_url(self.proxy_url, ca_certs=DEFAULT_CA) as http:
+            r = http.request("GET", f"{self.https_url}/")
+            assert r.status == 200
+            assert_is_verified(http, proxy=False, target=True)
+
+    def test_is_verified_https_proxy_to_http_target(self) -> None:
+        with proxy_from_url(self.https_proxy_url, ca_certs=DEFAULT_CA) as https:
+            r = https.request("GET", f"{self.http_url}/")
+            assert r.status == 200
+            assert_is_verified(https, proxy=True, target=False)
+
+    def test_is_verified_https_proxy_to_https_target(self) -> None:
+        with proxy_from_url(self.https_proxy_url, ca_certs=DEFAULT_CA) as https:
+            r = https.request("GET", f"{self.https_url}/")
+            assert r.status == 200
+            assert_is_verified(https, proxy=True, target=True)
+
     def test_http_and_https_kwarg_ca_cert_data_proxy(self) -> None:
         with open(DEFAULT_CA) as pem_file:
             pem_file_data = pem_file.read()
@@ -131,7 +170,7 @@ class TestHTTPProxyManager(HTTPDummyProxyTestCase):
             hc2 = http.connection_from_host(self.http_host, self.http_port)
             conn = hc2._get_conn()
             try:
-                hc2._make_request(conn, "GET", "/")
+                hc2._make_request(conn, "GET", f"{self.http_url}/")
                 tcp_nodelay_setting = conn.sock.getsockopt(  # type: ignore[attr-defined]
                     socket.IPPROTO_TCP, socket.TCP_NODELAY
                 )
@@ -173,13 +212,21 @@ class TestHTTPProxyManager(HTTPDummyProxyTestCase):
             r = http.request("GET", f"{self.https_url}/")
             assert r.status == 200
 
+    @resolvesLocalhostFQDN()
+    def test_proxy_https_fqdn(self) -> None:
+        with proxy_from_url(self.proxy_url, ca_certs=DEFAULT_CA) as http:
+            r = http.request("GET", f"{self.https_url_fqdn}/")
+            assert r.status == 200
+
     def test_proxy_verified(self) -> None:
         with proxy_from_url(
             self.proxy_url, cert_reqs="REQUIRED", ca_certs=self.bad_ca_path
         ) as http:
-            https_pool = http._new_pool("https", self.https_host, self.https_port)
-            with pytest.raises(MaxRetryError) as e:
-                https_pool.request("GET", "/", retries=0)
+            with http._new_pool(
+                "https", self.https_host, self.https_port
+            ) as https_pool:
+                with pytest.raises(MaxRetryError) as e:
+                    https_pool.request("GET", "/", retries=0)
             assert isinstance(e.value.reason, SSLError)
             assert (
                 "certificate verify failed" in str(e.value.reason)
@@ -190,22 +237,26 @@ class TestHTTPProxyManager(HTTPDummyProxyTestCase):
             http = proxy_from_url(
                 self.proxy_url, cert_reqs="REQUIRED", ca_certs=DEFAULT_CA
             )
-            https_pool = http._new_pool("https", self.https_host, self.https_port)
-
-            conn = https_pool._new_conn()
-            assert conn.__class__ == VerifiedHTTPSConnection
-            https_pool.request("GET", "/")  # Should succeed without exceptions.
+            with http._new_pool(
+                "https", self.https_host, self.https_port
+            ) as https_pool2:
+                with contextlib.closing(https_pool._new_conn()) as conn:
+                    assert conn.__class__ == VerifiedHTTPSConnection
+                    https_pool2.request(
+                        "GET", "/"
+                    )  # Should succeed without exceptions.
 
             http = proxy_from_url(
                 self.proxy_url, cert_reqs="REQUIRED", ca_certs=DEFAULT_CA
             )
-            https_fail_pool = http._new_pool("https", "127.0.0.1", self.https_port)
-
-            with pytest.raises(
-                MaxRetryError, match="doesn't match|IP address mismatch"
-            ) as e:
-                https_fail_pool.request("GET", "/", retries=0)
-            assert isinstance(e.value.reason, SSLError)
+            with http._new_pool(
+                "https", "127.0.0.1", self.https_port
+            ) as https_fail_pool:
+                with pytest.raises(
+                    MaxRetryError, match="doesn't match|IP address mismatch"
+                ) as e:
+                    https_fail_pool.request("GET", "/", retries=0)
+                assert isinstance(e.value.reason, SSLError)
 
     def test_redirect(self) -> None:
         with proxy_from_url(self.proxy_url) as http:
@@ -631,10 +682,10 @@ class TestHTTPProxyManager(HTTPDummyProxyTestCase):
 
 
 @pytest.mark.skipif(not HAS_IPV6, reason="Only runs on IPv6 systems")
-class TestIPv6HTTPProxyManager(IPv6HTTPDummyProxyTestCase):
+class TestIPv6HTTPProxyManager(IPv6HypercornDummyProxyTestCase):
     @classmethod
     def setup_class(cls) -> None:
-        HTTPDummyProxyTestCase.setup_class()
+        super().setup_class()
         cls.http_url = f"http://{cls.http_host}:{int(cls.http_port)}"
         cls.http_url_alt = f"http://{cls.http_host_alt}:{int(cls.http_port)}"
         cls.https_url = f"https://{cls.https_host}:{int(cls.https_port)}"
